import sys
from csv import reader

from ajgu.storage.bsddb3 import NoTxnBSDDB3
from ajgu import GraphDatabase


def main():
    storage = NoTxnBSDDB3('./db', (4, 0), (8, 0))
    graph = GraphDatabase(storage)

    # create indices
    with graph.transaction() as txn:
        txn.vertex.index('movieid')
        txn.vertex.index('userid')
        txn.edge.index('tag')

    print('Starting load of `movies.csv` in `_db/`')
    with open('./ml-latest-small/movies.csv', 'r') as f:
        next(reader(f))  # skip heading
        with graph.transaction() as txn:
            for movieid, title, genres in reader(f):
                movie = txn.vertex.create('movie')
                movie.props['movieid'] = movieid
                movie.props['title'] = title
                movie.props['genres'] = genres.split('|')

    print('Starting load of `ratings.csv` in `_db/`')
    with open('./ml-latest-small/ratings.csv', 'r') as f:
        next(reader(f))  # skip heading
        for userid, movieid, rating, timestamp in reader(f):
            with graph.transaction() as txn:
                # get or create user
                user = txn.vertex.filter('userid', userid).first()
                if not user:
                    user = txn.vertex.create('user')
                    user.props['userid'] = userid
                # create edge between movieand user
                movie = txn.vertex.filter('movieid', movieid).first()
                edge = txn.edge.create(user, 'rating', movie)

                edge.props['value'] = float(rating)
                edge.props['timestamp'] = float(timestamp)

    print('Starting load of `tags.csv` in `_db/`')
    with open('./ml-latest-small/tags.csv', 'r') as f:
        next(reader(f))  # skip heading
        with graph.transaction() as txn:
            for userid, movieid, tag, timestamp in reader(f):
                movie = txn.vertex.filter('movieid', movieid).first()
                user = txn.vertex.filter('userid', userid).first()
                edge = txn.edge.create(user, 'tag', movie)
                edge.props['value'] = tag
                edge.props['timestamp'] = float(timestamp)

    print('closing')
    graph.close()
    sys.exit()


if __name__ == '__main__':
    main()
